package ch.zhaw.projekt2.turnierverwaltung;

import java.io.Serializable;
import java.util.logging.Logger;

/**
 * Class that represents a person and holds its attributes
 */
public class Person implements Serializable {
    private final String NAME_MATCHING_REGEX = "[a-zA-Z]{1,20}";
    private final String PHONE_MATCHING_REGEX = "[+]?[0-9]*";

    private String name;
    private String firstName;
    private String phoneNumber;

    private static final Logger logger = Logger.getLogger(Person.class.getCanonicalName());


    /**
     * Constructor to initialize a person, also in charge of checking if name and phone number follow a valid format
     *
     * @param firstName   of the new person
     * @param name        of the new person
     * @param phoneNumber of the new person
     * @throws InvalidNameException        thrown when the name or firstname does not follow the valid format
     * @throws InvalidPhoneNumberException thrown when the number does not follow a valid format
     */
    public Person(String firstName, String name, String phoneNumber) throws InvalidNameException, InvalidPhoneNumberException {
        logger.finer("Trying to initialize a new person with name: " + name + ", first name: "
                + firstName + ", Phone number: " + phoneNumber);

        setFirstName(firstName);
        setName(name);
        setPhoneNumber(phoneNumber);
        logger.fine("Successfully created a new Person with Name: " + name + ", Firstname: "
                + firstName + ", Phone number: " + phoneNumber);
    }

    /**
     * Method to return the name of a Person
     *
     * @return the name as String
     */
    public String getName() {
        logger.fine("Returning name of:" + name);
        return name;
    }

    /**
     * Method to set the name of a Person
     *
     * @param name of the person to be set
     */
    public void setName(String name) throws InvalidNameException {
        if (!name.matches(NAME_MATCHING_REGEX)) {
            logger.warning("Name: " + name + ", is not in a valid format");
            throw new InvalidNameException("The Last name is Invalid");
        }
        logger.fine("Setting new name of " + this + " with: " + name);
        this.name = name;
    }

    public String getFirstName() {
        logger.fine("Returning firstname of " + this + "that is: " + firstName);
        return firstName;
    }

    /**
     * Method to set the first name of a Person
     *
     * @param firstName the person to be set
     */
    public void setFirstName(String firstName) throws InvalidNameException {
        if (!firstName.matches(NAME_MATCHING_REGEX)) {
            logger.warning("First name: " + firstName + ", is not in a valid format");
            throw new InvalidNameException("The First name is Invalid.");
        }
        logger.fine("Setting new first name of " + this + " with: " + firstName);
        this.firstName = firstName;
    }

    /**
     * Method used to get the Phone number of a Person
     *
     * @return phone number of the person
     */
    public String getPhoneNumber() {
        logger.fine("returning phone number of " + this + "that is: " + phoneNumber);
        return phoneNumber;
    }

    /**
     * Method used to set the Phone number of a Person
     *
     * @param phoneNumber to be set
     */
    public void setPhoneNumber(String phoneNumber) throws InvalidPhoneNumberException {
        if (!phoneNumber.matches(PHONE_MATCHING_REGEX)) {
            logger.warning("Phone number: " + phoneNumber + ", is not in a valid format");
            throw new InvalidPhoneNumberException("The entered Phone Number is invalid.");
        }
        logger.fine("Setting new phone number of " + this + " with: " + phoneNumber);
        this.phoneNumber = phoneNumber;
    }

    /**
     * Exception, that is used when a Phone number is in an invalid format.
     */
    public static class InvalidPhoneNumberException extends Exception {

        public InvalidPhoneNumberException(String errorMessage) {
            super(errorMessage);
        }

    }

    /**
     * Overridden toString method to be Return the name in Format "Firstname Name".
     *
     * @return Returning the Name in format "Firstname Name" as String
     */
    @Override
    public String toString() {
        return firstName + " " + name;
    }

}
